@@ -1,7 +1,7 @@
 /* -*- buffer-read-only: t -*- vi: set ro:
  *
  * Prototypes for autoopts
- * Generated Sun Mar 31 09:54:35 PDT 2013
+ * Generated Sat Dec  6 13:03:27 PST 2014
  */
 #ifndef AUTOOPTS_PROTO_H_GUARD
 #define AUTOOPTS_PROTO_H_GUARD 1
@@ -92,20 +92,22 @@ static void
 munge_str(char * txt, tOptionLoadMode mode);
 
 static void
-loadOptionLine(
-    tOptions *  opts,
-    tOptState * opt_state,
-    char *      line,
-    tDirection  direction,
-    tOptionLoadMode   load_mode );
+load_opt_line(tOptions * opts, tOptState * opt_state, char * line,
+              tDirection direction, tOptionLoadMode load_mode );
 
 /*
  *  Extracted from makeshell.c
  */
+static noreturn void
+option_exits(int exit_code);
+
+static noreturn void
+ao_bug(char const * msg);
+
 static void
 fserr_warn(char const * prog, char const * op, char const * fname);
 
-static void
+static noreturn void
 fserr_exit(char const * prog, char const * op, char const * fname);
 
 /*
@@ -133,7 +135,7 @@ optionSort(tOptions * opts);
  *  Extracted from stack.c
  */
 static void
-addArgListEntry(void** ppAL, void* entry);
+addArgListEntry(void ** ppAL, void * entry);
 
 /*
  *  Extracted from usage.c
@@ -141,7 +143,4 @@ addArgListEntry(void** ppAL, void* entry);
 static void
 set_usage_flags(tOptions * opts, char const * flg_txt);
 
-static void
-ao_bug(char const * msg);
-
 #endif /* AUTOOPTS_PROTO_H_GUARD */
